/*
 * fmsn
 * Copyright (C) 2003  Choiday
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Choiday
 * day@smileday.net
 */

#include "network.h"
#include "global.h"

gint
Socket_connect (gchar *host, gint port)
{
	gint m_sock;
	struct sockaddr_in m_addr;
	struct hostent	*servhost;
	
	if ((servhost = gethostbyname(host)) == NULL)
	{
		g_print ("bad hostname!\n");
		return -1;
	}	

	memset ( &m_addr, 0, sizeof ( m_addr ) );
	
	m_sock = Socket_create();
	
	m_addr.sin_family = AF_INET;
	m_addr.sin_port = htons ( port );
	m_addr.sin_addr = *((struct in_addr *)servhost->h_addr);
  	
  	if ( errno == EAFNOSUPPORT ) return -1;

  	gint status = connect ( m_sock, (struct sockaddr * ) &m_addr, sizeof ( m_addr ) );

  	if ( status == 0 ) 
		return m_sock;
	else
		return -1;
}	


gint is_valid(gint m_sock) { 
	return m_sock != -1; 
}


gint 
Socket_create()
{
  gint m_sock = socket ( AF_INET, SOCK_STREAM,  0 );

  if ( ! is_valid(m_sock) )
    return -1;


  // TIME_WAIT - argh
  gint on = 1;
  if ( setsockopt ( m_sock, SOL_SOCKET, SO_REUSEADDR, ( const gchar* ) &on, sizeof ( on ) ) == -1 )
    return -1;


  return m_sock;

}

gint 
Socket_send ( gint m_sock, const gchar *fmt, ...)
{
	gint status;
	gchar buffer[MAXRECV];
	va_list args;

	va_start(args, fmt);
	vsprintf(buffer, fmt, args);
	va_end(args);

	status = send ( m_sock, buffer, strlen(buffer), MSG_NOSIGNAL );
	
	if ( status == -1 )
		return -1;
	else
		return 1;
}

gint 
Socket_recv ( gint m_sock, gchar *str )
{
  gchar buf[MAXRECV+1];

  strcpy(str,"");

  memset ( buf, 0, MAXRECV + 1 );

  gint status = recv ( m_sock, buf, MAXRECV, 0 );

  if ( status == -1 )
    {
      g_print("status == -1   errno = %d in Socket_recv\n",errno);
      return -1;
    }
  else if ( status == 0 )
    {
      return -1;
    }
  else
    {
      strcpy(str, buf);
      return status;
    }
}

gint Send( GIOChannel *channel, const gchar *fmt, ...)
{
	guint len;
	GError *error = NULL;
	gchar buffer[MAXRECV];
	va_list args;

	va_start(args, fmt);
	vsprintf(buffer, fmt, args);
	va_end(args);
	
	g_io_channel_write_chars(channel, buffer, strlen(buffer), &len, &error);
	
	g_print("%s\n", buffer);
	
	if(error != NULL) {
		g_print("Could not write to channel: %s\n", error->message);
		g_error_free(error);
		return -1;
	}
	return 1;
}
